# This file is part of Buildbot.  Buildbot is free software: you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Copyright Buildbot Team Members


from buildbot.test.fakedb.row import Row


class ChangeSource(Row):
    table = "changesources"

    id_column = 'id'
    hashedColumns = [('name_hash', ('name',))]

    def __init__(self, id=None, name='csname', name_hash=None):
        super().__init__(id=id, name=name, name_hash=name_hash)


class ChangeSourceMaster(Row):
    table = "changesource_masters"

    def __init__(self, changesourceid=None, masterid=None):
        super().__init__(changesourceid=changesourceid, masterid=masterid)



# This file is part of Buildbot.  Buildbot is free software: you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Copyright Buildbot Team Members
from twisted.cred import credentials
from twisted.internet import defer
from twisted.internet import reactor
from twisted.spread import pb

from buildbot.util import unicode2bytes


class Sender:
    def __init__(self, master, auth=('change', 'changepw'), encoding='utf8'):
        self.username = unicode2bytes(auth[0])
        self.password = unicode2bytes(auth[1])
        self.host, self.port = master.split(":")
        self.port = int(self.port)
        self.encoding = encoding

    @defer.inlineCallbacks
    def send(
        self,
        branch,
        revision,
        comments,
        files,
        who=None,
        category=None,
        when=None,
        properties=None,
        repository='',
        vc=None,
        project='',
        revlink='',
        codebase=None,
    ):
        if properties is None:
            properties = {}

        change = {
            'project': project,
            'repository': repository,
            'who': who,
            'files': files,
            'comments': comments,
            'branch': branch,
            'revision': revision,
            'category': category,
            'when': when,
            'properties': properties,
            'revlink': revlink,
            'src': vc,
        }

        # codebase is only sent if set; this won't work with masters older than
        # 0.8.7
        if codebase:
            change['codebase'] = codebase

        for key, value in change.items():
            if isinstance(value, bytes):
                change[key] = value.decode(self.encoding, 'replace')
        change['files'] = list(change['files'])
        for i, file in enumerate(change.get('files', [])):
            if isinstance(file, bytes):
                change['files'][i] = file.decode(self.encoding, 'replace')

        f = pb.PBClientFactory()
        d = f.login(credentials.UsernamePassword(self.username, self.password))
        reactor.connectTCP(self.host, self.port, f)

        remote = yield d
        yield remote.callRemote('addChange', change)
        yield remote.broker.transport.loseConnection()



# This file is part of Buildbot.  Buildbot is free software: you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Copyright Buildbot Team Members


def patch():
    """
    Patch startService and stopService so that they check the previous state
    first.

    (used for debugging only)
    """
    from twisted.application.service import Service

    old_startService = Service.startService
    old_stopService = Service.stopService

    def startService(self):
        assert not self.running, f"{self!r} already running"
        return old_startService(self)

    def stopService(self):
        assert self.running, f"{self!r} already stopped"
        return old_stopService(self)

    Service.startService = startService
    Service.stopService = stopService



# This file is part of Buildbot.  Buildbot is free software: you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Copyright Buildbot Team Members

from __future__ import annotations

from buildbot.process.properties import Properties
from buildbot.test.fake.state import State


class Change(State):
    project = ''
    repository = ''
    branch = ''
    category = ''
    codebase = ''
    properties: dict | Properties = {}

    def __init__(self, **kw):
        super().__init__(**kw)
        # change.properties is a IProperties
        props = Properties()
        props.update(self.properties, "test")
        self.properties = props



# This file is part of Buildbot.  Buildbot is free software: you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Copyright Buildbot Team Members


from buildbot.db import base
from buildbot.util.sautils import hash_columns


class TagsConnectorComponent(base.DBConnectorComponent):
    def findTagId(self, name):
        tbl = self.db.model.tags
        name_hash = hash_columns(name)
        return self.findSomethingId(
            tbl=tbl,
            whereclause=(tbl.c.name_hash == name_hash),
            insert_values={"name": name, "name_hash": name_hash},
        )



# This file is part of Buildbot.  Buildbot is free software: you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Copyright Buildbot Team Members

"""add pause_reason column to workers table

Revision ID: 064
Revises: 063

"""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = '064'
down_revision = '063'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column("workers", sa.Column("pause_reason", sa.Text, nullable=True))


def downgrade():
    op.drop_column("workers", "pause_reason")
